/*******************************************************************************
 *	BdsC.h	Produced by Bidl
 *******************************************************************************
 */

#ifndef BDSC_H
#define BDSC_H 1

#include <stdlib.h>
#include <stdint.h>
#include <Boap.h>
#include <BString.h>
#include <BList.h>
#include <BArray.h>
#include <BdsD.h>


namespace Bds {
	const BUInt32 apiVersion = 0;

	/// @class DataAccess
	/// @brief This is the Data Access API interface.
	/// @details This object provides the set of API functions that make RPC network calls to a BdsServer.
	/// It provides the basic and restricted user orientated read only, data access API.
	
	class DataAccess : public BoapClientObject {
	public:
			DataAccess(BString name = "");
		BError connect(BString user, BString password);	///< Provides user/password information for secure connection
		BError validateUser(BString user, BString email);	///< Checks the user given name or email
		BError setUser(BString user, BString email);	///< Sets user to given name or email
		BError setUserReal();	///< Sets user back to real user
		BError getVersion(BString& version, BString& name);	///< Gets the software version and server name
		// Information functions
		BError userGetFromId(BUInt32 id, User& user);	///< Get user info given user ID
		BError userGet(User& user);	///< Get user info
		BError userSet(User user);	///< Set user info
		BError userGetGroups(BList<BString >& groups);	///< Get list of groups the user belongs to
		BError groupGetList(BList<Group >& groups);	///< Get list of Groups
		BError networkGetList(BList<Network >& networks);	///< Get list of Networks
		BError stationGetList(Selection sel, BList<Station >& stations);	///< Get list of Stations
		BError channelGetList(Selection sel, BList<Channel >& channels);	///< Get list of Channels
		BError sourceGetList(BList<Source >& sources);	///< Get list of Sources
		BError sourcePriorityGetList(BList<SourcePriority >& sourcePrioritys);	///< Get list of SourcePriorities
		BError dataFileGetList(Selection sel, BList<DataFileInfo >& dataFile);	///< Get list of DataFiles
		BError dataChannelGetList(Selection sel, BList<DataChannel >& dataChannel);	///< Get list of DataChannels
		BError channelInstrumentGetList(Selection sel, BList<ChannelInstrument >& channelInstruments);	///< Get list of Instruments
		BError digitiserGetList(Selection sel, BList<Digitiser >& digitisers);	///< Get list of Digitisers
		BError digitiserGet(BUInt32 id, Digitiser& digitiser);	///< Get a Digitiser object given its ID
		BError sensorGetList(Selection sel, BList<Sensor >& sensors);	///< Get list of Sensors
		BError sensorGet(BUInt32 id, Sensor& sensor);	///< Get a Sensor object given its ID
		BError calibrationGetList(Selection sel, BList<Calibration >& calibrations);	///< Get list of Calibrations
		BError responseGetList(Selection sel, BList<Response >& responses);	///< Get list of Responces
		BError locationGetList(Selection sel, BList<Location >& locations);	///< Get list of Station Locations
		// Selections
		BError getSelectionInfo(SelectionGroup group, Selection selectionIn, SelectionInfo& selectionInfo);	///< Get information on possible selections. Use in GUI programs to list options available.
		BError getSelections(SelectionGroup group, Selection selectionIn, Selection& selectionOut);	///< Get selection list
		BError dataAvailability(Selection selection, BUInt32 num, BArray<DataAvailChan >& dataAvailChans);	///< Return availability for data matching the given selection parameters
		BError dataSearch(Selection selection, DataInfo& dataInfo);	///< Search for data matching the given selection parameters
		BError dataGetChannelInfo(DataInfo dataInfo, ChannelInfos& channelInfos);	///< Return the channel MetaData in structured form
		BError dataOpen(DataInfo dataInfo, BString mode, BString format, BUInt32 flags, DataHandle& dataHandle);	///< Open a data file
		BError dataGetInfo(DataHandle dataHandle, BUInt32 infoExtra, DataInfo& dataInfo);	///< Get information on the data file
		BError dataGetNotes(DataHandle dataHandle, BList<Note >& notes);	///< Get notes on the data file
		BError dataGetWarnings(DataHandle dataHandle, BList<BString >& warnings);	///< Get information on the data file
		BError dataSeekBlock(DataHandle dataHandle, BUInt32 channel, BUInt32 segment, BTimeStamp time, BUInt32& blockNumber);	///< Searches for a data block matching the time given
		BError dataGetBlock(DataHandle dataHandle, BUInt32 channel, BUInt32 segment, BUInt32 blockNumber, DataBlock& data);	///< Return a block of data
		BError dataClose(DataHandle dataHandle, BError error, BInt32 del);	///< Close a file
		BError dataFormattedRead(DataHandle dataHandle, BUInt32 number, BArray<BUInt8 >& data);	///< Read the raw data from the file
		BError dataFormattedGetLength(DataHandle dataHandle, BUInt64& length);	///< Read the raw data from the file
		BError noteGetList(Selection sel, BList<Note >& notes);	///< Return a list of Notes
		BError noteUpdate(BInt32 append, Note note, BUInt32& id);	///< Add or update a Note
		BError noteWriteDocument(BUInt32 id, BString format, BArray<BUInt8 > data);	///< Given a Note write a document associated with it
		BError noteReadDocument(BUInt32 id, BString& format, BArray<BUInt8 >& data);	///< Read a document associated with a Note
		BError logUpdate(BInt32 append, Log log, BUInt32& id);	///< Add or update a Log item
		BError logAppend(BString type, BUInt32 priority, BString subSystem, BString title, BString description);	///< Append a log item
		// Management functions
		BError modeSet(Mode mode, Mode& previousMode);	///< Changes the system mode from Master to slave
		BError modeSnapshotPause(BInt32 on);	///< Enables/disables backup synchronisation pause
		BError clean(CleanOptions cleanOptions);	///< Cleans the system logs and Changes information
		BError databaseBackup(BString& ref);	///< Backup the database
		// Information functions
		BError statisticsGet(BDict<BString >& info);	///< Get a list of system statistics
		BError dataFormatGetList(BList<DataFormat >& formats);	///< Get list of data formats
	private:
	};
	/// @class DataAddAccess
	/// @brief This is the DataAdd Access API interface.
	/// @details This object provides the set of API functions that make RPC network calls to a BdsServer.
	/// It provides the basic and restricted user orientated read only, data access API along with the ability
	/// to import data. It will normally be used by data import client programs.
	class DataAddAccess : public BoapClientObject {
	public:
			DataAddAccess(BString name = "");
		BError connect(BString user, BString password);	///< Provides user/password information
		BError validateUser(BString user, BString email);	///< Checks the user given name or email
		BError setUser(BString user, BString email);	///< Sets user to given name or email
		BError setUserReal();	///< Sets user back to real user
		BError getVersion(BString& version, BString& name);	///< Gets the software version and server name
		// Information functions
		BError userGetFromId(BUInt32 id, User& user);	///< Get user info given user ID
		BError userGet(User& user);	///< Get user info
		BError userSet(User user);	///< Set user info
		BError userGetGroups(BList<BString >& groups);	///< Get list of groups the user belongs to
		BError groupGetList(BList<Group >& groups);	///< Get list of Groups
		BError networkGetList(BList<Network >& networks);	///< Get list of Networks
		BError stationGetList(Selection sel, BList<Station >& stations);	///< Get list of Stations
		BError channelGetList(Selection sel, BList<Channel >& channels);	///< Get list of Channels
		BError sourceGetList(BList<Source >& sources);	///< Get list of Sources
		BError sourcePriorityGetList(BList<SourcePriority >& sourcePrioritys);	///< Get list of SourcePriorities
		BError dataFileGetList(Selection sel, BList<DataFileInfo >& dataFile);	///< Get list of DataFiles
		BError dataChannelGetList(Selection sel, BList<DataChannel >& dataChannel);	///< Get list of DataChannels
		BError channelInstrumentGetList(Selection sel, BList<ChannelInstrument >& channelInstruments);	///< Get list of Instruments
		BError digitiserGetList(Selection sel, BList<Digitiser >& digitisers);	///< Get list of Channels
		BError digitiserGet(BUInt32 id, Digitiser& digitiser);	///< Get a Digitiser object given its ID
		BError sensorGetList(Selection sel, BList<Sensor >& sensors);	///< Get list of Sensors
		BError sensorGet(BUInt32 id, Sensor& sensor);	///< Get a Sensor object given its ID
		BError calibrationGetList(Selection sel, BList<Calibration >& calibrations);	///< Get list of Calibrations
		BError responseGetList(Selection sel, BList<Response >& responses);	///< Get list of Responses
		BError locationGetList(Selection sel, BList<Location >& locations);	///< Get list of Station Locations
		// Selections
		BError getSelectionInfo(SelectionGroup group, Selection selectionIn, SelectionInfo& selectionInfo);	///< Get information on possible selections. Use in GUI programs to list options available.
		BError getSelections(SelectionGroup group, Selection selectionIn, Selection& selectionOut);	///< Get selection list
		BError dataAvailability(Selection selection, BUInt32 num, BArray<DataAvailChan >& dataAvailChans);	///< Return availability for data matching the given selection parameters
		BError dataSearch(Selection selection, DataInfo& dataInfo);	///< Search for data matching the given selection parameters
		BError dataGetChannelInfo(DataInfo dataInfo, ChannelInfos& channelInfos);	///< Return the channel MetaData in structured form
		BError dataOpen(DataInfo dataInfo, BString mode, BString format, BUInt32 flags, DataHandle& dataHandle);	///< Open a data file
		BError dataGetInfo(DataHandle dataHandle, BUInt32 infoExtra, DataInfo& dataInfo);	///< Get information on the data file
		BError dataGetNotes(DataHandle dataHandle, BList<Note >& notes);	///< Get notes on the data file
		BError dataGetWarnings(DataHandle dataHandle, BList<BString >& warnings);	///< Get information on the data file
		BError dataSeekBlock(DataHandle dataHandle, BUInt32 channel, BUInt32 segment, BTimeStamp time, BUInt32& blockNumber);	///< Searches for a data block matching the time given
		BError dataGetBlock(DataHandle dataHandle, BUInt32 channel, BUInt32 segment, BUInt32 blockNumber, DataBlock& data);	///< Return a block of data
		BError dataSetInfo(DataHandle dataHandle, DataInfo dataInfo);	///< Set the info when writing to a file
		BError dataPutBlock(DataHandle dataHandle, DataBlock data);	///< Send a block of data
		BError dataClose(DataHandle dataHandle, BError error, BInt32 del);	///< Close a file
		BError dataFormattedRead(DataHandle dataHandle, BUInt32 number, BArray<BUInt8 >& data);	///< Read the raw data from the stream
		BError dataFormattedGetLength(DataHandle dataHandle, BUInt64& length);	///< The total length in bytes of the formated data.
		BError noteGetList(Selection sel, BList<Note >& notes);	///< Return a list of Notes
		BError noteUpdate(BInt32 append, Note note, BUInt32& id);	///< Add or update a Note
		BError noteWriteDocument(BUInt32 id, BString format, BArray<BUInt8 > data);	///< Given a Note write a document associated with it
		BError noteReadDocument(BUInt32 id, BString& format, BArray<BUInt8 >& data);	///< Read a document associated with a Note
		BError logUpdate(BInt32 append, Log log, BUInt32& id);	///< Append a log item					///< Add or update a Log item
		BError logAppend(BString type, BUInt32 priority, BString subSystem, BString title, BString description);	///< Append a log item
		// Management functions
		BError modeSet(Mode mode, Mode& previousMode);	///< Changes the system mode from Master to slave
		BError modeSnapshotPause(BInt32 on);	///< Enables/disables backup synchronisation pause
		BError clean(CleanOptions cleanOptions);	///< Cleans the system logs and Changes information
		BError databaseBackup(BString& ref);	///< Backup the database
		// Information functions
		BError statisticsGet(BDict<BString >& info);	///< Get a list of system statistics
		BError dataFormatGetList(BList<DataFormat >& formats);	///< Get list of data formats
	private:
	};
	/// @class AdminAccess
	/// @brief This is the AdminAccess Access API interface.
	/// @details This object provides the set of API functions that make RPC network calls to a BdsServer.
	/// It provides the full unresiticed data access API allowring all data read and write operations to be performed.
	class AdminAccess : public BoapClientObject {
	public:
			AdminAccess(BString name = "");
		BError connect(BString user, BString password);	///< Provides user/password information
		BError validateUser(BString user, BString email);	///< Checks the user given name or email
		BError setUser(BString user, BString email);	///< Sets user to given name or email
		BError setUserReal();	///< Sets user back to real user
		BError getVersion(BString& version, BString& name);	///< Gets the software version and server name
		// User functions
		BError userGetList(BList<User >& users);	///< Get list of Users
		BError userUpdate(BInt32 append, User user, BUInt32& id);	///< Update or append a user entry
		BError userDelete(BUInt32 id);	///< Delete a user entry
		BError userGetFromId(BUInt32 id, User& user);	///< Get user info given user ID
		BError userGet(User& user);	///< Get user info
		BError userSet(User user);	///< Set user info
		BError userGetGroups(BList<BString >& groups);	///< Get list of groups the user belongs to
		BError groupGetList(BList<Group >& groups);	///< Get list of Groups
		BError groupUpdate(BInt32 append, Group group, BUInt32& id);	///< Update or append a group entry
		BError groupDelete(BUInt32 id);	///< Delete a group entry
		BError accessGroupGetList(BList<AccessGroup >& accessGroups);	///< Get list of AccessGroups
		BError accessGroupUpdate(BInt32 append, AccessGroup group, BUInt32& id);	///< Update or append an AccessGroup entry
		BError accessGroupDelete(BUInt32 id);	///< Delete an AccessGroup entry
		// Selections
		BError getSelectionInfo(SelectionGroup group, Selection selectionIn, SelectionInfo& selectionInfo);	///< Get information on possible selections. Use in GUI programs to list options available.
		BError getSelections(SelectionGroup group, Selection selectionIn, Selection& selectionOut);	///< Get selection list
		BError networkGetList(BList<Network >& networks);	///< Get list of Networks
		BError networkUpdate(BInt32 append, Network network, BUInt32& id);	///< Add or update a Network entry
		BError networkDelete(BUInt32 id);	///< Delete a Network entry
		BError stationGetList(Selection sel, BList<Station >& stations);	///< Get list of Stations
		BError stationUpdate(BInt32 append, Station station, BUInt32& id);	///< Add or update a Station entry
		BError stationDelete(BUInt32 id);	///< Delete a Station entry
		BError locationGetList(Selection sel, BList<Location >& locations);	///< Get list of Station Locations
		BError locationUpdate(BInt32 append, Location location, BUInt32& id);	///< Add or update a Station Location entry
		BError locationDelete(BUInt32 id);	///< Delete a Station Location entry
		BError channelGetList(Selection sel, BList<Channel >& channels);	///< Get list of Channels
		BError channelGet(BUInt32 id, Channel& channel);	///< Get a channel
		BError channelUpdate(BInt32 append, Channel channel, BUInt32& id);	///< Add or update a Channel entry
		BError channelDelete(BUInt32 id);	///< Delete a Channel entry
		BError sourceGetList(BList<Source >& sources);	///< Get list of Sources
		BError sourceUpdate(BInt32 append, Source source, BUInt32& id);	///< Add or update a Source entry
		BError sourceDelete(BUInt32 id);	///< Delete a Source entry
		BError sourcePriorityGetList(BList<SourcePriority >& sourcePrioritys);	///< Get list of SourcePriorities
		BError sourcePriorityUpdate(BInt32 append, SourcePriority sourcePriority, BUInt32& id);
		BError sourcePriorityDelete(BUInt32 id);
		BError channelInstrumentGetList(Selection sel, BList<ChannelInstrument >& channelInstruments);	///< Get list of Channel Instruments
		BError channelInstrumentUpdate(BInt32 append, ChannelInstrument channelInstrument, BUInt32& id);	///< Add or update a Instrument entry
		BError channelInstrumentDelete(BUInt32 id);	///< Delete an Instrument entry
		BError digitiserGetList(Selection sel, BList<Digitiser >& digitisers);	///< Get list of Digitisers
		BError digitiserGet(BUInt32 id, Digitiser& digitiser);	///< Get a Digitiser object given its ID
		BError digitiserUpdate(BInt32 append, Digitiser digitiser, BUInt32& id);	///< Add or update a Digitiser entry
		BError digitiserDelete(BUInt32 id);	///< Delete a Digitiser entry
		BError sensorGetList(Selection sel, BList<Sensor >& sensors);	///< Get list of Sensors
		BError sensorGet(BUInt32 id, Sensor& sensor);	///< Get a Sensor object given its ID
		BError sensorUpdate(BInt32 append, Sensor sensor, BUInt32& id);	///< Add or update a Sensor entry
		BError sensorDelete(BUInt32 id);	///< Delete a Sensor entry
		BError calibrationGetList(Selection sel, BList<Calibration >& calibrations);	///< Get list of Calibrations
		BError calibrationUpdate(BInt32 append, Calibration calibration, BUInt32& id);	///< Add or update a Calibration entry
		BError calibrationDelete(BUInt32 id);	///< Delete a Calibration entry
		BError responseGetList(Selection sel, BList<Response >& responses);	///< Get list of Responses
		BError responseUpdate(BInt32 append, Response response, BUInt32& id);	///< Add or update a Response entry
		BError responseDelete(BUInt32 id);	///< Delete a Response entry
		BError dataFileGetList(Selection sel, BList<DataFileInfo >& dataFile);	///< Get list of DataFiles
		BError dataFileUpdate(BInt32 append, DataFileInfo dataFile, BUInt32& id);	///< Add or update a DataFile entry
		BError dataFileDelete(BUInt32 id);	///< Delete a DataFile entry
		BError dataChannelGetList(Selection sel, BList<DataChannel >& dataChannel);	///< Get list of DataChannels
		BError dataChannelUpdate(BInt32 append, DataChannel dataChannel, BUInt32& id);	///< Add or update a DataChannel entry
		BError dataChannelDelete(BUInt32 id);	///< Delete a DataChannel entry
		BError dataAvailability(Selection selection, BUInt32 num, BArray<DataAvailChan >& dataAvailChans);	///< Return availability for data matching the given selection parameters
		BError dataSearch(Selection selection, DataInfo& dataInfo);	///< Search for data matching the given selection parameters
		BError dataGetChannelInfo(DataInfo dataInfo, ChannelInfos& channelInfos);	///< Return the channel MetaData in structured form
		BError dataOpen(DataInfo dataInfo, BString mode, BString format, BUInt32 flags, DataHandle& dataHandle);	///< Open a data file
		BError dataGetInfo(DataHandle dataHandle, BUInt32 infoExtra, DataInfo& dataInfo);	///< Get information on the data file
		BError dataGetNotes(DataHandle dataHandle, BList<Note >& notes);	///< Get notes on the data file
		BError dataGetWarnings(DataHandle dataHandle, BList<BString >& warnings);	///< Get information on the data file
		BError dataGetBlock(DataHandle dataHandle, BUInt32 channel, BUInt32 segment, BUInt32 blockNumber, DataBlock& data);	///< Return a block of data
		BError dataSeekBlock(DataHandle dataHandle, BUInt32 channel, BUInt32 segment, BTimeStamp time, BUInt32& blockNumber);	///< Searches for a data block matching the time given
		BError dataSetInfo(DataHandle dataHandle, DataInfo dataInfo);	///< Set the info when writing to a file
		BError dataPutBlock(DataHandle dataHandle, DataBlock data);	///< Send a block of data
		BError dataClose(DataHandle dataHandle, BError error, BInt32 del);	///< Close a file
		BError dataFormattedRead(DataHandle dataHandle, BUInt32 number, BArray<BUInt8 >& data);	///< Read the raw data from the file
		BError dataFormattedGetLength(DataHandle dataHandle, BUInt64& length);	///< Read the raw data from the file
		// Change/Logging functions
		BError changeGroupStart(ChangeGroup changeGroup);	///< Start a new ChangeGroup when making a set of changes to the BDS's database
		BError changeGroupEnd();	///< End a ChangeGroup
		BError changeGroupGetList(ListRange range, BList<ChangeGroup >& changeGroups);	///< Return a list of ChangeGroups
		BError changeGroupDelete(BTimeStamp beforeDate, BString type, BInt32 empty);	///< Delete a ChangeGroup
		BError changeGetListNumber(BUInt32 id, BUInt32& number);	///< Get the number of changes in a ChangeGroup
		BError changeGetList(BUInt32 id, ListRange range, BList<Change >& changes);	///< Get a list of Changes
		BError changeDelete(BTimeStamp beforeDate, BString type);	///< Delete a Change
		BError noteGetList(Selection sel, BList<Note >& notes);	///< Get a list of Notes
		BError noteUpdate(BInt32 append, Note note, BUInt32& id);	///< Add or update a Note
		BError noteDelete(BUInt32 id);	///< Delete a Note
		BError noteWriteDocument(BUInt32 id, BString format, BArray<BUInt8 > data);	///< Given a Note write a document associated with it
		BError noteReadDocument(BUInt32 id, BString& format, BArray<BUInt8 >& data);	///< Read a document associated with a Note
		BError logGetList(LogSelect sel, BList<Log >& logs);	///< Get list of log entries
		BError logUpdate(BInt32 append, Log log, BUInt32& id);	///< Add or Update a Log item
		BError logDelete(BUInt32 id);	///< Delete a Log item
		BError logAppend(BString type, BUInt32 priority, BString subSystem, BString title, BString description);	///< Append a log item
		// Information functions
		BError statisticsGet(BDict<BString >& info);	///< Get a list of system statistics
		BError dataFormatGetList(BList<DataFormat >& formats);	///< Get list of supported data formats
		// Management functions
		BError transactionStart();	///< Starts a set of transactions
		BError transactionEnd(BInt32 abort);	///< Ends a set of transactions
		BError modeSet(Mode mode, Mode& previousMode);	///< Changes the system mode from Master to slave
		BError modeSnapshotPause(BInt32 on);	///< Enables/disables backup synchronisation pause
		BError clean(CleanOptions cleanOptions);	///< Cleans the system logs and Changes information
		BError databaseBackup(BString& ref);	///< Backup the database
		BError databaseRestore(BString ref, BString type);	///< Restore the database
		// Low level functions
		BError sqlQuery(BString query, BList<BDict<BString > >& result);	///< A low level SQL access function
	private:
	};
}
#endif
